import * as React from "react";
import { LayerElevations } from "./Layer";
import { TypeOptions } from "react-toastify/dist/types";
export interface AlertProps extends React.HTMLAttributes<HTMLDivElement> {
    /** If used, a close button will be attached to the alert box. */
    onRequestClose?: () => void;
    /** Primary text */
    title?: string;
    /** Secondary text */
    subtitle?: string | React.ReactNode;
    /** A unique id used for accessibility purposes */
    id?: string;
    elevation?: LayerElevations;
    /** Optionally render children if a title is not specified. Used for custom alerts. */
    children?: React.ReactNode;
    /** Changes the icon and colour of the alert. */
    intent?: TypeOptions;
    component?: React.ElementType<any>;
    type?: "polite" | "assertive";
}
/**
 * Use an alert to inform users of important information.
 * To display an alert in a toast notification, use the toast module.
 */
export declare const Alert: ({ children, title, id, subtitle, component, elevation, onRequestClose, intent, ...other }: AlertProps) => import("@emotion/react/jsx-runtime").JSX.Element;
interface Props extends React.HTMLAttributes<HTMLDivElement> {
    children?: React.ReactNode;
    /** Primary text */
    title: string;
    /** Secondary text */
    subtitle?: string | React.ReactNode;
    /** A unique id used for accessibility purposes */
    id?: string;
}
/**适配新的'react-toastify'包， 只需要里面信息。
 *若children非空 && title=null？ 等于没起任何效果 用的。
 */
export declare const AlertContent: ({ children, title, id, subtitle, ...other }: Props) => import("@emotion/react/jsx-runtime").JSX.Element;
export {};
